package com.seeyon.cip.spi.user.auth.thirdauth;

import com.seeyon.cip.spi.user.auth.thirdauth.constants.ThirdauthSsoAuthConstants;
import com.seeyon.cip.spi.user.auth.thirdauth.handler.ThirdauthSsoAuthHandler;
import com.seeyon.ctp.user.annotation.CtpUserChannelRouter;
import com.seeyon.ctp.user.api.sso.CtpUserSsoAuthProviderService;
import com.seeyon.ctp.user.dto.CtpUserSpiLoginUserInfoDto;
import com.seeyon.ctp.user.dto.CtpUserSpiServerEnvDto;
import com.seeyon.ctp.user.dto.CtpUserSpiThirdTokenDto;
import com.seeyon.ctp.user.dto.CtpUserSpiUserDto;
import com.seeyon.ctp.user.enums.CtpUserSpiAuthTokenEnum;
import com.seeyon.ctp.user.exception.CtpUserSpiSsoException;
import com.seeyon.ctp.user.util.CtpUserSpiUtils;
import lombok.extern.slf4j.Slf4j;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 统一三方认证 SPI 扩展接口, 实现类需要增加 @CtpUserChannelRouter 注解<br/>
 * 注意：<br/>
 * 类中不需要加 @Service 方法， 也不能使用 @Autowired 注解， 无法获取到对应实现。需要使用 CtpUserSpiUtils#getInstance() 方法获取实例<br/>
 * request 对象获取： CtpUserSpiUtils#getRequest()<br/>
 * nacos 等配置信息获取： CtpUserSpiUtils#getPropertyByName(java.lang.String)，通过配置的 key 直接获取
 */
@Slf4j
@CtpUserChannelRouter("test")
public class ThirdauthSsoAuthProviderService implements CtpUserSsoAuthProviderService {

    /**
     * [按需使用] 服务域名、前端路由信息
     */
    private CtpUserSpiServerEnvDto ctpUserSpiServerEnvDto;

    /**
     * [按需使用] 通过工具类，获取依赖类的实例， 注：不能使用 @Autowired 注解
     */
    private ThirdauthSsoAuthHandler thirdauthSsoAuthHandler = CtpUserSpiUtils.getInstance(ThirdauthSsoAuthHandler.class);

    /**
     * [按需实现] 获取v8服务环境信息, 包括 服务域名、前端地址路由 信息。 按需使用
     *
     * @param ctpUserSpiServerEnvDto 服务域名、前端路由信息
     */
    @Override
    public void setServerEnv(CtpUserSpiServerEnvDto ctpUserSpiServerEnvDto) {
        // 测试日志打印
        log.info("{} enter setServerEnv", ThirdauthSsoAuthConstants.THIRD_TYPE);
        this.ctpUserSpiServerEnvDto = ctpUserSpiServerEnvDto;
    }

    /**
     * [按需实现] 登录模式映射关系
     *
     * @return key: 设备类型， value： 支持的登录模式
     */
    @Override
    public Map<String, CtpUserSpiAuthTokenEnum[]> mapSupportsAuthTokenEnum() {
        Map<String, CtpUserSpiAuthTokenEnum[]> loginTokenEnumMap = new HashMap<>();
        loginTokenEnumMap.put("WEB", new CtpUserSpiAuthTokenEnum[]{CtpUserSpiAuthTokenEnum.SPI_SSO});
        // 支持的终端登录方式， 按需将下面注释放开
        // 桌面端
        // loginTokenEnumMap.put("PC", new CtpUserSpiAuthTokenEnum[]{CtpUserSpiAuthTokenEnum.USERNAME_PASSWORD,CtpUserSpiAuthTokenEnum.PHONE_VERIFY_CODE});
        // 手机端
        // loginTokenEnumMap.put("MOBILE", new CtpUserSpiAuthTokenEnum[]{CtpUserSpiAuthTokenEnum.USERNAME_PASSWORD,CtpUserSpiAuthTokenEnum.PHONE_VERIFY_CODE});
        return loginTokenEnumMap;
    }

    /**
     * 三方认证 sso 登录地址
     *
     * @param request
     * @param encodeRedirectUrl url encode 后的重定向地址，三方认证登录成功后的跳转地址
     *                          encodeRedirectUrl: http://dev.seeeyon.com/thirdauth-login/aaa?url=/main/protal
     * @return 三方认证中心登录地址
     */
    @Override
    public String getSsoLoginUrl(HttpServletRequest request, String encodeRedirectUrl) {
        String thirdLoginUrl = "http://xxx.sso.com/login?redirectUrl=" + encodeRedirectUrl;
        log.info("跳转到三方认证中心地址:{}", thirdLoginUrl);
        return thirdLoginUrl;
    }


    /**
     * 三方认证中心登录成功后， 追加的请求参数key， 如 code、ticket 等
     *
     * @return 追加的请求参数key
     */
    @Override
    public String getRequestParaKey() {
        return "code";
    }

    /**
     * [按需实现] 手机端、pc 端、pad 端登录使用，通过用户名密码等信息认证并获取登录名。<br/>
     * 手机端直接调用向后端入参内容。<br/>
     * 如果登录异常提示想要提示在页面上，则使用 CtpUserSpiSsoException 抛出，提示内容为 CtpUserSpiSsoException 里的 message 的值
     *
     * @param userLoginName           登录名
     * @param extData                 扩展信息：包含密码信息、设备信息等
     * @param ctpUserSpiAuthTokenEnum 认证方式类型
     * @return CtpUserSpiLoginUserInfoDto 三方用户信息 DTO
     * @throws CtpUserSpiSsoException 自定义显示登录失败的异常信息
     */
    @Override
    public CtpUserSpiLoginUserInfoDto getUserLoginInfoForMobile(String userLoginName, Map<String, Object> extData, CtpUserSpiAuthTokenEnum ctpUserSpiAuthTokenEnum) throws CtpUserSpiSsoException {
        return null;
    }

    /**
     * [按需实现] 通过手机号发送短信功能<br/>
     * 使用三方认证中心进行短信验证码登录时使用
     *
     * @param mobile 手机号
     * @return true: 短信发送成功， false： 短信发送失败， null： 不发送短信
     */
    @Override
    public Boolean sendVerifyCode(String mobile) {
        // 通过手机号 调用三方认中心 进行短信发送操作
        return null;
    }

    /**
     * 从三方认证接口中获取用户信息
     *
     * @param request
     * @param encodeRedirectUrl url encode 后的重定向地址，三方认证登录成功后的跳转地址
     * @return CtpUserSpiLoginUserInfoDto 三方用户信息 DTO
     * @throws CtpUserSpiSsoException 三方认证登录失败时， 由 v8 自定义页面显示的异常信息
     */
    @Override
    public CtpUserSpiLoginUserInfoDto getUserLoginInfo(HttpServletRequest request, String encodeRedirectUrl) throws CtpUserSpiSsoException {
        // 通过配置拿到三方 clientId
        String clientId = ThirdauthSsoAuthConstants.NACOS_CLIENT_ID;
        // 调用内部方法进行逻辑处理
        thirdauthSsoAuthHandler.test();
        // 调用三方认证接口，获取用户信息
        // 获取三方认证中心登录后，追加的 code 的值
        String code = request.getParameter(getRequestParaKey());
        // 通过 code 获取 token
        // eg. String token = getTokenByCode(code);
        // 通过 token 获取用户登录名
        // eg. String loginName = getLoginNameByToken(token);

        // // [按需实现] 如果需要缓存三方认证中心的 token，则需要封装下面对象
        // CtpUserSpiThirdTokenDto ctpUserSpiThirdTokenDto = CtpUserSpiThirdTokenDto.builder()
        //         .thirdToken("123456") // 三方认证中心的 token
        //         .expired(3600L) // 过期时间_秒 可空，如果为空则默认 7 天
        //         .extData(new HashMap<>()) // [按需实现] 扩展参数， 下面刷新三方 token 方法 refreshThirdauthTokenByConfig 中的 thirdauthTokenConfigMap 会将这个扩展参数传入
        //         .build();

        return CtpUserSpiLoginUserInfoDto.builder()
                // 用于登录的用户信息， 下面五个参数只选择一下赋值即可， 赋值的参数的值要与 v8系统中的对应的值一致
                .thirdLoginName("zhangsan")
                // .thirdUserId("123456")
                // .thirdMobile("13900000001")
                // .thirdUserEmail("abc@abc.com")
                // .thirdUserCode("usercode")
                //用于登录的用户信息， 上面五个参数只选择一下赋值即可

                // 如果用户有租户条件
                // .tenantType("code").tenantData(tenantCode) // or .tenantType("id").tenantData("-1")

                // .thirdUserInfoJson("{\"name\":\"张三\"}") // [按需实现] 待缓存的三方用户信息， 下面登录方法 thirdauthSsoLogout 中将这个参数传入
                // .ctpUserSpiThirdTokenDto(ctpUserSpiThirdTokenDto) // 如果需要缓存并刷新三方 token 信息，则封装这个参数
                .build();
    }

    /**
     * [按需实现] 通过登录名登录成功后， 会存在多个用户信息， 自己选择哪个用户登录
     *
     * @param ctpUserSpiUserDtoList
     * @return
     */
    @Override
    public Long rebuildCtpUserDtoList(List<CtpUserSpiUserDto> ctpUserSpiUserDtoList) {
        return CtpUserSsoAuthProviderService.super.rebuildCtpUserDtoList(ctpUserSpiUserDtoList);
    }

    /**
     * [按需实现] 三方认证中心 token 中否保存到 cookie中，如果需要， 则返回 cookie 的 key
     *
     * @return 保存三方 token 的 cookie 的 key 的值
     */
    @Override
    public String get4aTokenKey() {
        return CtpUserSsoAuthProviderService.super.get4aTokenKey();
    }

    /**
     * [按需实现] 刷新三方认证中心的 token
     *
     * @param thirdToken          三方认证中心 token
     * @param thirdTokenConfigMap 三方认证中心 token 配置
     * @return
     */
    @Override
    public CtpUserSpiThirdTokenDto refreshThirdTokenByConfig(String thirdToken, Map<String, Object> thirdTokenConfigMap) {
        return CtpUserSsoAuthProviderService.super.refreshThirdTokenByConfig(thirdToken, thirdTokenConfigMap);
    }

    /**
     * [按需实现] 三方登出后台逻辑
     *
     * @param request
     * @param thirdUserInfoJson
     */
    @Override
    public void thirdSsoLogout(HttpServletRequest request, String thirdUserInfoJson) {
        CtpUserSsoAuthProviderService.super.thirdSsoLogout(request, thirdUserInfoJson);
    }

    /**
     * [按需实现] 三方登出重定向地址， 三方认证中心可通过此地址进行用户登录操作，如果没有此接口，则不用实现
     *
     * @return url encode 后的登出后的重定向地址
     */
    @Override
    public String getEncodeLogoutRedirectUrl() {
        return CtpUserSsoAuthProviderService.super.getEncodeLogoutRedirectUrl();
    }
}