package com.seeyon.cip.spi.user.auth.thirdauth.example;

import com.seeyon.cip.spi.user.auth.thirdauth.constants.ThirdauthSsoAuthConstants;
import com.seeyon.ctp.user.annotation.CtpUserChannelRouter;
import com.seeyon.ctp.user.api.sso.CtpUserSsoAuthProviderService;
import com.seeyon.ctp.user.dto.CtpUserSpiLoginUserInfoDto;
import com.seeyon.ctp.user.exception.CtpUserSpiSsoException;
import lombok.extern.slf4j.Slf4j;

import javax.servlet.http.HttpServletRequest;

/**
 * <pre>
 * 名词解释:
 * v8目标地址（受保护的资源地址）: 要登录v8 后可以看到的地址， 如 /main/portal、 待办地址 等
 * 三方登录地址： 如果v8系统未登录，需要跳转到三方认证中心提供登录页面的地址
 * 三方登录code： 三方登录成功后， 会为用户生成随机数， 用于查询用户信息使用， 这个随机值会在登录成功后通过 url 重定向传递给 v8
 * 三方登录code对应key： 追加的 三方登录code的key，一般为 code， 但也有 ticket、 tokencode 等， 根据三方认证中心接口规范实现
 * v8用户查询条件： 通过 用户id / 用户 code/ 手机号/ 登录名 / 邮箱 查询用户信息。 从左向右，有且只取第一个非空值做为条件
 *
 * 简单示例及操作流程：
 * 1. web 浏览器端打开 v8 地址， 如果未登录， 跳转到三方登录地址 http://simple.sso.com?redirectUrl=v8中间页面地址（里面包含 [v8目标地址]）
 * 2. 三方登录成功后， 重定向三方登录地址中的 redirectUrl 地址， 并追加 [三方登录code]
 * 3. v8 接收到 [三方登录code]， 并通过 code 向三方认证中心查询用户信息
 * 4. 通过三方返回的用户信息，封装成 [v8用户查询条件], 查询 v8 用户信息
 * 5. 如果 v8 存在该用户，登录成功 生成 accessToken, 并成功跳转到 [v8目标地址]
 *
 * demo示例具体元素：
 * v8目标地址： http://dev.seeyonv8.com/main/portal
 * 三方登录地址： http://simple.sso.com
 * v8用户查询条件： 登录名 为 zhangsan
 *
 * </pre>
 */
@Slf4j
@CtpUserChannelRouter(ThirdauthSsoAuthConstants.THIRD_TYPE) // 注： 注解必有要有
public class A_SimpleAuthProviderService implements CtpUserSsoAuthProviderService {

    /**
     * 1. 配置 三方登录 code 对应 key： 即 追加随机值的变量名， 一般为 code, 按需实现
     */
    @Override
    public String getRequestParaKey() {
        return "code";
    }

    /**
     * <pre>
     * 2. 拼装三方登录地址 http://simple.sso.com， 里面会携带三方登录成功后的重定向地址
     * encodeRedirectUrl 为 v8 登录中间页面地址+v8目标地址, 并完成 url encode 编码
     *
     * 示例：
     * encodeRedirectUrl = https%3A%2F%2Fdev.seeyonv8.com%2Flogin%2Fthird-auth%2Fcode%3FredirectUri%3Dhttps%253A%252F%252Fdev.seeyonv8.com%252Fmain%252Fportal
     * url decode 后： https://dev.seeyonv8.com/login/third-auth/code?redirectUri=https%3A%2F%2Fdev.seeyonv8.com%2Fmain%2Fportal
     *
     * https://dev.seeyonv8.com/login/third-auth/code 为中间页地址， /code 为 getRequestParaKey() 方法中 配置的 key， 如果配置的是 ticket, 则这里为 https://dev.seeyonv8.com/login/third-auth/ticket
     * 如果 v8 登录成功后， 会跳转到 redirectUri=https%3A%2F%2Fdev.seeyonv8.com%2Fmain%2Fportal
     * 再次 url decode 后， redirectUri=https://dev.seeyonv8.com/main/portal 为 v8 登录成功后的目标地址
     *
     * 如果 v8 未登录， 则会跳转到的三方认证地址为：
     * http://simple.sso.com?redirectUrl=https%3A%2F%2Fdev.seeyonv8.com%2Flogin%2Fthird-auth%2Fcode%3FredirectUri%3Dhttps%253A%252F%252Fdev.seeyonv8.com%252Fmain%252Fportal
     * 如果三方认证中心登录成功后，会重定向到 redirectUrl 地址， 并追加 [三方登录code]
     * 重定向跳转到 https://dev.seeyonv8.com/login/third-auth/code?redirectUri=https%3A%2F%2Fdev.seeyonv8.com%2Fmain%2Fportal&code=123456
     * </pre>
     */
    @Override
    public String getSsoLoginUrl(HttpServletRequest request, String encodeRedirectUrl) {
        // 三方登录地址： http://simple.sso.com?redirectUrl=xxx
        // ctp-user 会拼装好三方登录成功后的重定向地址， 并通过 url encode 编码，默认情况下只需要追加到 redirectUrl 后面即可
        String thirdLoginUrl = "http://simple.sso.com?redirectUrl=" + encodeRedirectUrl;
        log.info("跳转到三方认证中心地址:{}", thirdLoginUrl);
        return thirdLoginUrl;
    }


    /**
     * 从三方认证接口中获取用户信息
     *
     * @param request
     * @param encodeRedirectUrl url encode 后的重定向地址，三方认证登录成功后的跳转地址
     * @return CtpUserSpiLoginUserInfoDto 三方用户信息 DTO
     * @throws CtpUserSpiSsoException 三方认证登录失败时， 由 v8 自定义页面显示的异常信息
     */
    @Override
    public CtpUserSpiLoginUserInfoDto getUserLoginInfo(HttpServletRequest request, String encodeRedirectUrl) throws CtpUserSpiSsoException {
        // 获取认证中心互信的配置信息，如 ak / sk 等
        String clientId = ThirdauthSsoAuthConstants.NACOS_CLIENT_ID;

        // 从地址中获取三方登录成功后的 code
        String code = request.getParameter(getRequestParaKey());
        // 通过 code 一次或多次请求获取到用户信息（如 oauth2 下， 通过code 获取token， 通过token 获取用户信息）
        // eg. String token = getTokenByCode(code);
        // 通过用户信息获取到登录名
        String loginName = "zhangsan";


        // 无租户条件返回的用户信息
return CtpUserSpiLoginUserInfoDto.builder()
                .thirdLoginName(loginName)
                .build();

//        // 有租户条件返回的用户信息
//        String tenantCode = "cop";
//        CtpUserSpiLoginUserInfoDto loginUserInfoHasTenant = CtpUserSpiLoginUserInfoDto.builder()
//                .thirdLoginName(loginName)
//                .tenantType("code").tenantData(tenantCode) // or .tenantType("id").tenantData("-1")
//                .build();
//
//        return null; // 按客户场景选择登录的用户是否需要有租户条件， 如果有，则返回 loginUserInfoHasTenant， 否则返回 loginUserInfoIgnoreTenant
    }
}