package com.seeyon.cip.spi.user.auth.thirdauth.example;

import com.seeyon.cip.spi.user.auth.thirdauth.constants.ThirdauthSsoAuthConstants;
import com.seeyon.ctp.user.annotation.CtpUserChannelRouter;
import com.seeyon.ctp.user.api.sso.CtpUserSsoAuthProviderService;
import com.seeyon.ctp.user.constant.CtpUserSpiConstants;
import com.seeyon.ctp.user.dto.CtpUserSpiLoginUserInfoDto;
import com.seeyon.ctp.user.enums.CtpUserSpiAuthTokenEnum;
import com.seeyon.ctp.user.exception.CtpUserSpiSsoException;
import lombok.extern.slf4j.Slf4j;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.Map;

/**
 * <pre>
 * 多端多设备登录示例
 * mobile / pc / pad 端认证登录配置
 * </pre>
 */
@Slf4j
@CtpUserChannelRouter(ThirdauthSsoAuthConstants.THIRD_TYPE) // 注： 注解必有要有
public class B_MultiAuthProviderService implements CtpUserSsoAuthProviderService {

    /**
     * 同 A_SimpleAuthProviderService 配置
     */
    @Override
    public String getRequestParaKey() {
        return "code";
    }

    /**
     * 同 A_SimpleAuthProviderService 配置
     */
    @Override
    public String getSsoLoginUrl(HttpServletRequest request, String encodeRedirectUrl) {
        // 三方登录地址： http://simple.sso.com?redirectUrl=xxx
        // ctp-user 会拼装好三方登录成功后的重定向地址， 并通过 url encode 编码，默认情况下只需要追加到 redirectUrl 后面即可
        String thirdLoginUrl = "http://simple.sso.com?redirectUrl=" + encodeRedirectUrl;
        log.info("跳转到三方认证中心地址:{}", thirdLoginUrl);
        return thirdLoginUrl;
    }

    /**
     * 同 A_SimpleAuthProviderService 配置
     * web 端登录三方认证中心成功后， 获取用户信息方法
     */
    @Override
    public CtpUserSpiLoginUserInfoDto getUserLoginInfo(HttpServletRequest request, String encodeRedirectUrl) throws CtpUserSpiSsoException {
        return CtpUserSpiLoginUserInfoDto.builder()
                .thirdLoginName("zhangsan")
                .build();
    }

    /**
     * <pre>
     * 手机端、PC 端、Pad 端 登录三方认证中心成功后， 获取用户信息方法
     * 入参中： userLoginName v8 登录页面中的登录名， extData 为 v8 登录页面中传递过来的参数， 包含明文密码、设备信息等信息。 CtpUserSpiAuthTokenEnum为登录类型，一般为账号密码方式登录
     * </pre>
     */
    @Override
    public CtpUserSpiLoginUserInfoDto getUserLoginInfoForMobile(String userLoginName, Map<String, Object> extData, CtpUserSpiAuthTokenEnum ctpUserSpiAuthTokenEnum) throws CtpUserSpiSsoException {
        // 通过账号、密码调用三方认证中心， 通过一次或多次请求获取到用户信息，并取出 v8用户查询条件， 如 loginName

        // 通过用户信息获取到登录名
        String loginName = "zhangsan";

        // 返回内容格式同 A_SimpleAuthProviderService
        return CtpUserSpiLoginUserInfoDto.builder()
                .thirdLoginName(loginName)
                .build();
    }

    /**
     * <pre>
     * 这里配置了 web 端、手机端、PC 端、Pad 端 登录方式
     * 其中 web端使用三方认证中心登录页面进行登录， 手机端、PC 端、Pad端使用 v8 登录页面进行登录
     * 注： 手机端、PC 端、Pad端 需要三方认证中心支持账号密码透传到认证中心并获取用户信息的接口，如果三方认证中心不支持则无法实现对应端使用三方认证中心登录的情况
     * </pre>
     */
    @Override
    public Map<String, CtpUserSpiAuthTokenEnum[]> mapSupportsAuthTokenEnum() {
        Map<String, CtpUserSpiAuthTokenEnum[]> loginTokenEnumMap = new HashMap();
        // web 端使用 三方登录页面，通过重定向方式进行登录打开 v8 目标页
        loginTokenEnumMap.put(CtpUserSpiConstants.DeviceType.WEB, new CtpUserSpiAuthTokenEnum[]{CtpUserSpiAuthTokenEnum.USERNAME_PASSWORD});
        // 手机端、pc 端、 pad 端使用 v8 登录页， 三方认证中心的账号密码登录
        loginTokenEnumMap.put(CtpUserSpiConstants.DeviceType.MOBILE, new CtpUserSpiAuthTokenEnum[]{CtpUserSpiAuthTokenEnum.USERNAME_PASSWORD});
        loginTokenEnumMap.put(CtpUserSpiConstants.DeviceType.PC, new CtpUserSpiAuthTokenEnum[]{CtpUserSpiAuthTokenEnum.USERNAME_PASSWORD});
        loginTokenEnumMap.put(CtpUserSpiConstants.DeviceType.PAD, new CtpUserSpiAuthTokenEnum[]{CtpUserSpiAuthTokenEnum.USERNAME_PASSWORD});
        return loginTokenEnumMap;
    }
}